/*
 * Pnote.c
 *
 * Routine to print correct note and accidental given the
 * midi note value and track.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mtm.h"

//	in the table below, & and # indicate that the note should
//	appear with that accidental.
//	+ and - indicate that the note is sharp or flat but this
//	is part of the major scale for the current key, and so
//	should be suppressed unles it follows some other accidental
//
//
#if 1
static char *scales[16];

void loadNoteList(char *path) {
  FILE *in = fopen(path, "r");
  char buff[256], *p;
  int i = 0;

  if (in == NULL) {
    fprintf(stderr, "mtm: error - unable to load note list data from %s\n",
	    path);
    perror("fopen");
    exit(1);
  }
  while (fgets(buff, 256, in) != NULL) {
    p = buff;
    while (*p == ' ' || *p == '\t')
      p++;
    if (p[0] == '/' || p[0] == '\n')
      continue;			// ignore comments and blanks
    p[24] = 0;
    if (i > 14) {
      fprintf(stderr, "mtm: error - too many note list lines in %s\n", path);
      exit(1);
    }
    scales[i++] = strdup(p);
  }
  if (i < 14) {
    fprintf(stderr, "mtm: error - not enough note list lines in %s\n", path);
    exit(1);
  }
  fclose(in);
}

#else
static char *scales[] = {
/* C C#D D#E F F#G G#A A#B  */
  "cnd-dne-f-fng-gna-anb-c-",		/* 7& C& major */
  "cnd-dne-enf g-gna-anb-c-",		/* 6& G& major */
  "c d-dne-enf g-gna-anb-bn",		/* 5& D& major */
  "c d-dne-enf f#g a-anb-bn",		/* 4& A& major */
  "c c#d e-enf f#g a-anb-bn",		/* 3& E& major */
  "c c#d e-enf f#g g#a b-bn",		/* 2& B& Major */
  "c c#d e&e f f#g g#a b-bn",		/* 1& F  major */
  "c c#d d#e f f#g g#a a#b ",		/* 0& C  major */
  "c c#d e&e fnf+g a&a b&b ",		/* 1# G  major */
  "cnc+d e&e fnf+g a&a b&b ",		/* 2# D  major */
  "cnc+d e&e fnf+gng+a b&b ",		/* 3# A  major */
  "cnc+dnd+e fnf+gng+a b&b ",		/* 4# E  major */
  "cnc+dnd+e fnf+gng+ana+b ",		/* 5# B  major */
  "cnc+dnd+ene+f+gng+ana+b ",		/* 6# F# major */
  "b+c+dnd+ene+f+gng+ana+bn",		/* 7# C# major */
};
#endif

extern FILE *out;

void setkeysig(Trackp t, int key) {
  int i;
  char *p;
  
  key += 7;
  if (key < 0 || key > 14)
    {
      fprintf(stderr, "mtm: key outside range -7..+7\n");
      exit(1);
    }
  p = scales[key];
  for (i=0; i<128; i++) {
    if (p[0] == 0)
      p = scales[key];
    t->name[i] = p[0];
    t->acc[i] = p[1];
    t->accp[i] = 0;
    p += 2;
  }
}

void resetaccs(Trackp t) {
  int i;
  for (i=0; i<128; i++)
    t->accp[i] = ' ';
}

void getPitch(Trackp t, Eventp np) {
  int nval = np->pitch;
  int o = goctave(t, nval);
  int n = t->name[nval];
  int a = t->acc[nval];
  int z = 7 * o + (n - 'a');

  if (t->accp[z] == ' ') {
    if (a == '+' || a == '-')
      t->accp[z] = ' ';
    else
      t->accp[z] = a;
  }
  else if (t->accp[z] == a)
    a = ' ';
  else if (a == ' ') {
    a = 'n';
    t->accp[z] = a;
  }
  else if (a == '-') {
    a = '&';
    t->accp[z] = ' ';
  }
  else if (a == '+') {
    a = '#';
    t->accp[z] = ' ';
  }
  np->oct  = o;
  np->note = n;
  np->acc  = a;
}

void putNote(Trackp t, Eventp np) {
  fputc(np->note, out);
  if (np->acc == ' ' || (np->flags & F_TIEB) != 0)
    return;
  if (np->acc == '-' || np->acc == '+')
    return;
  fputc(np->acc, out);
}
